#!/usr/bin/env python3

import numpy as np

def swap_rows(A, row1, row2):
    """Swap the rows row1 and row2 of A"""
    temp = np.copy(A[row1,:])
    A[row1,:] = A[row2,:]
    A[row2,:] = temp
    return A


def subtract_scaled(A, dst, src, scale):
    """Subtract scale times the row src from the row dst of A"""
    # tupel (m,n) mit m anzahl rows und n anzahl cols
    x = A.shape
    for i in range(0,x[1]):
        A[dst][i] = A[dst][i] - A[src][i] * scale
    return A


def pivot_index(A, column):
    """Compute the row index of the maximum element of A in this column, excluding all elements above the diagonal of A."""
    maxi = A[column][column]
    row = column
    for i in range (column, A.shape[0]):
        if abs(A[i][column]) > abs(maxi):
            maxi = A[i][column]
            row = i
    return row


def lu_decompose(A):
    """Return (P, L, U), such that A = P @ L @ U"""
    n = A.shape[0]
    #col = A.shape[1]
    P = np.eye(n)
    L = np.zeros((n,n))
    U = np.zeros((n,n))
    for j in range (0, n-1):
        pi = pivot_index(A,j)
        P = P@swap_rows(np.eye(n),pi,j)
        A = swap_rows(A,pi,j)
        for i in range (j+1, n):
            A[i,j] = A[i,j]/A[j,j]
            for k in range (j+1, n):
                A[i,k] -= A[i,j] * A[j,k]

    for j in range (0,n):
        L[j,j] = 1;
        for i in range (0,n):
            if i <= j:
                U[i,j] = A[i,j]
            else:
                L[i,j] = A[i,j]
    return (P,L,U)


def forward_substitute(L, b):
    """Return y such that L @ y = b"""
    col = L.shape[1]
    row = L.shape[0]
    y = np.zeros((col,1))
    for i in range (0,row):
        y[i] = b[i]
        for j in range(0,i):
            y[i] = y[i] - L[i][j] * y[j]
        y[i] = y[i] / L[i][i]
    return y


def backward_substitute(R, y):
    """Return x such that R @ x = y"""
    col = R.shape[1]
    row = R.shape[0]
    x = np.zeros((col,1))
    for i in range (row-1, -1,-1):
        x[i] = y[i]
        for j in range (i+1,col):
            x[i] = x[i] - R[i][j]*x[j]
        x[i] = x[i] / R[i][i]
    return x


def linsolve(A, *bs):
    """Return (x1, ..., xn), such that A @ xk = bs[k]"""
    a = []
    P,L,U = lu_decompose(A)
    for b in bs:
        x = backward_substitute(U, forward_substitute(L,P@b))
        a.append(x)
    return tuple(a)


def main():
    # Hier kann beliebiger Testcode stehen, der bei der Korrektur ignoriert wird
    print(linsolve(np.array([[ 1.]]), np.array([[ 0.]])))
    print(linsolve(np.array([[  1.,   1.,   1.,   1.],[  1.,   2.,   3.,   4.],[  1.,   3.,   6.,  10.],[  1.,   4.,  10.,  20.]]),np.array([[ 0.],[ 0.],[ 0.],[ 0.]]), np.array([[ 3.],[ 1.],[ 4.],[1.]])))


if __name__ == "__main__": main()

