#!/usr/bin/env python3

import numpy as np
from scipy.misc import imread, imshow, imsave
import scipy.ndimage as ndimage


def stretch(image, hstretch, wstretch):
    """Resize given grayscale image to the given width and height."""
    return ndimage.zoom(image, (hstretch, wstretch))


def downsample(image, N = 1):
    """Select only every N-th column and row from image."""
    cols = image.shape[1]
    rows = image.shape[0]
    newcols = 0
    newrows = 0
    for i in range (0, cols, N):
        newcols += 1
    for i in range (0, rows, N):
        newrows += 1
    res = np.zeros((newrows, newcols))
    for i in range (0,newrows):
        for j in range (0,newcols):
            res[i][j] = image[i*N][j*N]
    return res


def moire_parameter():
    return 25 # return 5, 10, 15, 20, 25, 30, 35, or 40


def main():
    img = imread('moire.png', flatten=True)

    N = 20
    # imshow(img)
    # imshow(downsample(img, N))
    imshow(stretch(downsample(img, N), N, N))


if __name__ == "__main__": main()
