#!/usr/bin/env python3

import numpy as np
import numpy.linalg
import matplotlib.pyplot as plt


def interpolate_linearly(a, b):
    """Return an object of type numpy.poly1d withe degree 1 that passes through a and b."""
    t = (a[1] - b[1]) / (a[0] - b[0])
    g = a[1] - a[0] * t
    return np.poly1d([t, g])


def newton_matrix(X):
    """Setup the matrix of the LSE which is used to determine the coefficients
    of the Newton-basis.  X are the x-coordinates of the nodes which are
    used for interpolation."""
    n = len(X)
    res = np.zeros((n,n))
    for j in range (0,n):           # spalten
        for i in range (j,n):       # zeilen
            res[i][j] = 1
    for j in range (0,n):           # spalten
        for i in range (j,n):       # zeilen
            for k in range(0,j):
                res[i][j] = res[i][j] * (X[i]-X[k])
    return res


def newton_polynomial(C, X):
    """Take coefficients and interpolation point x-coordinates of the
Newton-polynomial and determine the corresponding interpolation polynomial."""
    assert len(C) == len(X)
    res = np.poly1d([0])
    for i in range (0, len(C)):
        q = np.poly1d([C[i]])
        for j in range (1,i+1):
            k = np.poly1d([X[j-1]], True)
            q = q * k
        res = res + q
    return res


def interpolating_polynomial(X,Y):
    """Determine the interpolating polynomial for the given NumPy arrays of x and y coordinates."""
    assert len(X) == len(Y)
    n = len(X)
    a = newton_matrix(X)
    C = np.zeros((n))
    for i in range (0,n):
        C[i] = Y[i]
        for j in range (0,i):
            C[i] = C[i] - C[j]*a[i][j]
        C[i] = C[i] / a[i][i]
    return newton_polynomial(C,X)


def interpolation_plot(X,Y):
    p = interpolating_polynomial(X, Y)
    px = np.arange(min(X)-0.1, max(X)+0.11, 0.01)
    plt.grid(True)
    plt.plot(X, Y, "o")
    plt.plot(px, p(px))
    plt.show()


def main():
    X = np.array([0, 1, 2,3])
    Y = np.array([-2.,3.,1.,2.])
#    interpolation_plot(X, Y)
    print(newton_polynomial(np.array([1., 2., 3.]), np.array([0., 1., 1.])))

    print("soll:")
    print(np.poly1d([3., -1., 1.]))

if __name__ == "__main__": main()
