#!/usr/bin/env python3

import numpy as np
import matplotlib.cm as cm
import matplotlib.pyplot as plt
import matplotlib.animation as animation


glider = np.array([[0,1,0],
                   [0,0,1],
                   [1,1,1]],
                  dtype=bool)


c10orthogonal = np.array([[0,1,1,0,0,1,1,0],
                          [0,0,0,1,1,0,0,0],
                          [0,0,0,1,1,0,0,0],
                          [1,0,1,0,0,1,0,1],
                          [1,0,0,0,0,0,0,1],
                          [0,0,0,0,0,0,0,0],
                          [1,0,0,0,0,0,0,1],
                          [0,1,1,0,0,1,1,0],
                          [0,0,1,1,1,1,0,0],
                          [0,0,0,0,0,0,0,0],
                          [0,0,0,1,1,0,0,0],
                          [0,0,0,1,1,0,0,0]],
                         dtype=bool)

def gamegrid(w, h, entities):
    grid = np.zeros((h,w), dtype=bool)
    for (entity, x, y) in entities:
        add_entity(grid, entity, x, y)
    return grid


def add_entity(grid, entity, x, y):
    iks = entity.shape[0]
    yps = entity.shape[1]
    for i in range(0,iks):
        for j in range(0,yps):
            grid[i+x][j+y] = entity[i][j]
    return grid

def next_step(grid):
    n = grid.shape[0]
    m = grid.shape[1]
    res = np.zeros((n,m), dtype=bool)
    for i in range (0,n):
        for j in range (0,m):
            neighbors = 0
            if grid[(i-1)%n][(j-1)%m] == 1: 
                neighbors = neighbors + 1
            if grid[(i-1)%n][(j+1)%m] == 1: 
                neighbors = neighbors + 1
            if grid[(i+1)%n][(j-1)%m] == 1: 
                neighbors = neighbors + 1
            if grid[(i+1)%n][(j+1)%m] == 1: 
                neighbors = neighbors + 1
            if grid[(i)%n][(j+1)%m] == 1: 
                neighbors = neighbors + 1
            if grid[(i+1)%n][(j)%m] == 1: 
                neighbors = neighbors + 1
            if grid[(i)%n][(j-1)%m] == 1: 
                neighbors = neighbors + 1
            if grid[(i-1)%n][(j)%m] == 1: 
                neighbors = neighbors + 1
            if grid[i][j] == 0:
                if neighbors == 3: res[i][j] = 1
            if grid[i][j] == 1:
                if neighbors < 2 or neighbors > 3: res[i][j] = 0
                else: res[i][j] = 1
    return res

def gameoflife(grid, steps=100):
    fig, ax = plt.subplots()
    mat = ax.matshow(grid, cmap=cm.gray_r)
    ani = animation.FuncAnimation(fig, lambda _: mat.set_data(next_step(grid)),
                                  frames=100,
                                  interval=50,
                                  blit=False)
    plt.show()


def main():
    grid = gamegrid(40, 40, [(glider, 13, 4),
                             (c10orthogonal, 25, 25)])
    gameoflife(grid)

if __name__ == "__main__": main()
